#include "svn_repos.h"
#include "svn_diff.h"
#include "apr_pools.h"

#define PERROR(err)  if (err!=SVN_NO_ERROR) { \
		printf("(%d) Error was: %s\n", __LINE__, err->message); \
        goto bail; \
	}

typedef struct sv_diff_baton_t sv_diff_baton_t;
struct sv_diff_baton_t
{
    /* file to compare */
    const char *path;
    /* Two revisions to compare */
    svn_revnum_t rev1;
    svn_revnum_t rev2;
    /* FS */
    svn_fs_t *fs;
    /* Private members */
    /* FS roots */
    svn_fs_root_t *fs_rev1;
    svn_fs_root_t *fs_rev2;
    /* streams */
    svn_stream_t *file_rev1;
    svn_stream_t *file_rev2;
};

static
    svn_error_t *sv_diff__fs_open(void *baton,
                                  svn_diff_datasource_e datasource)
{

}

static
    svn_error_t *sv_diff__fs_close(void *baton,
                                   svn_diff_datasource_e datasource)
{
    return SVN_NO_ERROR;
}

static
    svn_error_t *sv_diff__fs_get_next_token(apr_uint32_t * hash,
                                             void **token, void *baton,
                                             svn_diff_datasource_e datasource)
{
}

static
svn_error_t *
sv_diff__fs_token_compare(void *baton,
                             void *token1,
                             void *token2,
                             int *compare)
{
}

static
void
sv_diff__fs_token_discard(void *baton,
                             void *token)
{
}

static
void
sv_diff__fs_token_discard_all(void *baton)
{
}

static const svn_diff_fns_t svn_diff__fs_vtable = {
    sv_diff__fs_open,
    sv_diff__fs_close,
    sv_diff__fs_get_next_token,
    sv_diff__fs_token_compare,
    sv_diff__fs_token_discard,
    sv_diff__fs_token_discard_all
};

int main(int argc, char *argv[])
{
    apr_pool_t *p;
    svn_repos_t *repo;
    svn_fs_t *fs;
    svn_revnum_t last_rev;

    svn_error_t *err;

    const char *mode;
    int i;
    char argp[2] = "\0\0";

    apr_table_t *params;

    apr_pool_initialize();
    apr_pool_create(&p, NULL);

    params = apr_table_make(p, 5);

    for (i = 1; i < argc; i++) {
        if (argv[i][0] == '-' && strlen(argv[i]) > 1) {
            argp[0] = argv[i][1];
            argp[1] = argv[i][2];
        }
        else {
            switch (argp[0]) {
            case 'r':
                switch (argp[1]) {
                case '1':
                    apr_table_set(params, "rev1", argv[i]);
                    printf("Rev1; %s\n", argv[i]);
                    break;
                case '2':
                    apr_table_set(params, "rev2", argv[i]);
                    printf("Rev2; %s\n", argv[i]);
                    break;
                default:
                    apr_table_set(params, "repo", argv[i]);
                    printf("Repo; %s\n", argv[i]);
                    break;
                }
                break;
            case 'f':
                apr_table_set(params, "file", argv[i]);
                printf("File; %s\n", argv[i]);
                break;
            case 'm':
                apr_table_set(params, "mode", argv[i]);
                printf("Mode; %s\n", argv[i]);
                break;
            }
        }
    }

    printf("Opening Repository: %s\n", apr_table_get(params, "repo"));
    err = svn_repos_open(&repo, apr_table_get(params, "repo"), p);
    PERROR(err);

    fs = svn_repos_fs(repo);
    printf("Checking yougest.\n");
    err = svn_fs_youngest_rev(&last_rev, fs, p);
    PERROR(err);

    printf("Latest Revision is %" SVN_REVNUM_T_FMT ".\n", last_rev);

    mode = apr_table_get(params, "mode");
    if (mode == NULL)
        mode = "list";

    if (strcmp(mode, "list") == 0) {
        const char *file = apr_table_get(params, "file");
        const char *str_rev1 = apr_table_get(params, "rev1");
        apr_hash_t *dirents;
        apr_hash_index_t *dirwalk;
        svn_fs_root_t *fs_root;

        if (str_rev1 == NULL) {
            err = svn_fs_revision_root(&fs_root, fs, last_rev, p);
            PERROR(err);
        }
        else {
            err = svn_fs_revision_root(&fs_root, fs, apr_atoi64(str_rev1), p);
            PERROR(err);
        }

        if (file == NULL)
            file = "/";

        printf("Listing %s\n", file);
        err = svn_fs_dir_entries(&dirents, fs_root, file, p);
        PERROR(err);

        for (dirwalk = apr_hash_first(p, dirents);
             dirwalk != NULL; dirwalk = apr_hash_next(dirwalk)) {
            char *key;
            int klen;
            svn_fs_dirent_t *val;

            apr_hash_this(dirwalk, (void *) &key, &klen, (void **) &val);
            if (val->kind == svn_node_dir) {
                printf("   [%s] (Revision: )\n", key);
            }
            else {
                printf("   %s (Revision: )\n", key);
            }
        }
    }
    else if (strcmp(mode, "diff") == 0) {
        const char *file = apr_table_get(params, "file");
        const char *str_rev1 = apr_table_get(params, "rev1");
        const char *str_rev2 = apr_table_get(params, "rev2");
        svn_stream_t *stream_rev1, *stream_rev2;
        svn_fs_root_t *fs_root_rev1, *fs_root_rev2;
        svn_revnum_t rev1, rev2;
        svn_txdelta_stream_t *stream_delta;
        svn_txdelta_window_t *delta_win;

        if (str_rev1 == NULL) {
            printf("Must specify starting revision. (-r1 num)\n");
            goto bail;
        }
        else {
            rev1 = apr_atoi64(str_rev1);
        }
        if (str_rev2 == NULL) {
            printf("Assuming second revision %" SVN_REVNUM_T_FMT
                   " use (-r2 num)\n", last_rev);
            rev2 = last_rev;
        }
        else {
            rev2 = apr_atoi64(str_rev2);
        }
        printf("Diffing file %s.\n", file);
        printf("Grabbing Revision %" SVN_REVNUM_T_FMT ".\n", rev1);
        err = svn_fs_revision_root(&fs_root_rev1, fs, rev1, p);
        PERROR(err);

        err = svn_fs_file_contents(&stream_rev1, fs_root_rev1, file, p);
        PERROR(err);

        printf("Grabbing Revision %" SVN_REVNUM_T_FMT ".\n", rev2);
        err = svn_fs_revision_root(&fs_root_rev2, fs, rev2, p);
        PERROR(err);

        err = svn_fs_file_contents(&stream_rev2, fs_root_rev2, file, p);
        PERROR(err);

        printf("Diffing\n");
        svn_txdelta(&stream_delta, stream_rev1, stream_rev2, p);

        svn_txdelta_next_window(&delta_win, stream_delta, p);
        while (delta_win) {
            printf("");

            svn_txdelta_next_window(&delta_win, stream_delta, p);
        }
    }
  bail:
    apr_pool_terminate();
    return 0;
}
